@@ -6,7 +6,7 @@
 # @Version : 3.8.9 Python
 
 from PIL import Image, ImageDraw, ImageFont
-from typing import List, Tuple
+from typing import List, Tuple, Union
 import numpy as np
 from nonebot.typing import T_State
 from nonebot.adapters.onebot.v11 import Bot, MessageSegment, MessageEvent, GroupMessageEvent, Message
@@ -718,7 +718,7 @@ def draw_test(text: str, color: Tuple, save_path: Path, fontpath: Path):
 # 入侵流程-----------------------------------------------------------------
 
 
-def one_by_one(team: List[str], plants: List[str]) -> (List, int):
+def one_by_one(team: List[str], plants: List[str]) -> (List[MessageSegment], int):
     if not os.path.exists(PVZ_OUTPUT_PATH):
         os.makedirs(PVZ_OUTPUT_PATH)
     # 或去当前全部植物的对象
@@ -731,16 +731,8 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
     # 僵尸序号
     zcnt = 0
     # 入侵日志
-    log = [
-        {
-            "type": "node",
-            "data": {
-                "name": "疯狂戴夫",
-                "uin": "1298919732",
-                "content": "入！侵！开始！"
-            }
-        }
-    ]
+    log = []
+    log.append(MessageSegment.text("入！侵！开始！"))
     for zombie in team:
         # 全局时间
         time = 0
@@ -764,29 +756,7 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
         # 生成图片，写入日志
         lawn_pic([all_plants[x] for x in plants if x != "0"], cnt)
         zombie_pic(all_zombie[zombie], dist, cnt)
-        log.append(
-            {
-                "type": "node",
-                "data": {
-                    "name": "疯狂戴夫",
-                    "uin": "1298919732",
-                    "content": [
-                        {
-                            "type": "text",
-                            "data": {
-                                "text": "当前双方阵容"
-                            }
-                        },
-                        {
-                            "type": "image",
-                            "data": {
-                                "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
-                            }
-                        }
-                    ]
-                }
-            }
-        )
+        log.append(MessageSegment.text("当前双方阵容")+MessageSegment.image(f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"))
         # 图片计数序号递增
         cnt += 1
         # 当前僵尸循环，当僵尸死亡后退出，每次循环经过0.5秒
@@ -824,23 +794,7 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                     # 生成图片，打印日志
                     lawn_pic([all_plants[x] for x in plants if x != "0"], cnt)
                     zombie_pic(all_zombie[zombie], dist, cnt)
-                    log.append(
-                        {
-                            "type": "node",
-                            "data": {
-                                "name": "疯狂戴夫",
-                                "uin": "1298919732",
-                                "content": [
-                                    {
-                                        "type": "image",
-                                        "data": {
-                                            "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
-                                        }
-                                    }
-                                ]
-                            }
-                        }
-                    )
+                    log.append(MessageSegment.image(f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"))
                     cnt += 1
                 else:
                     fight_time += 0.5
@@ -849,21 +803,7 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                     if lawn_plants[tar].unjumpable == 1:
                         zb.jump = 0
                         zb.v = zb.v/2
-                        log.append({
-                            "type": "node",
-                            "data": {
-                                "name": "疯狂戴夫",
-                                "uin": "1298919732",
-                                "content": [
-                                    {
-                                        "type": "text",
-                                        "data": {
-                                            "text": f"时间{time},{zombie}被高坚果阻挡后无法起跳，撑杆掉落，速度减半"
-                                        }
-                                    },
-                                ]
-                            }
-                        })
+                        log.append(MessageSegment.text(f"时间{time},{zombie}被高坚果阻挡后无法起跳，撑杆掉落，速度减半"))
                     else:
                         # 距离递减
                         dist -= 1
@@ -874,75 +814,17 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                         if zb.v == 0.4:
                             zb.jump = 0
                             zb.v = 0.2
-                            log.append({
-                                "type": "node",
-                                "data": {
-                                    "name": "疯狂戴夫",
-                                    "uin": "1298919732",
-                                    "content": [
-                                        {
-                                            "type": "text",
-                                            "data": {
-                                                "text": f"时间{time},{zombie}被植物阻挡后起跳，撑杆掉落，速度减半"
-                                            }
-                                        },
-                                        {
-                                            "type": "image",
-                                            "data": {
-                                                "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
-                                            }
-                                        }
-                                    ]
-                                }
-                            }
-                            )
+                            log.append(MessageSegment.text(f"时间{time},{zombie}被植物阻挡后起跳，撑杆掉落，速度减半")+MessageSegment.image(f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"))
                         else:
                             # 如果是跳跳僵尸，则可以继续跳过，同时失去攻击机会
-                            log.append({
-                                "type": "node",
-                                "data": {
-                                    "name": "疯狂戴夫",
-                                    "uin": "1298919732",
-                                    "content": [
-                                        {
-                                            "type": "text",
-                                            "data": {
-                                                "text": f"时间{time},{zombie}被植物阻挡后起跳,同时企图继续跳过其他植物"
-                                            }
-                                        },
-                                        {
-                                            "type": "image",
-                                            "data": {
-                                                "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
-                                            }
-                                        }
-                                    ]
-                                }
-                            }
-                            )
+                            log.append(MessageSegment.text(f"时间{time},{zombie}被植物阻挡后起跳,同时企图继续跳过其他植物")+MessageSegment.image(f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"))
                         cnt += 1
                     # 判断当前目标
                     if tar > 0:
                         lawn_plants[tar - 1].hp -= zb.damage
-                        log.append({
-                            "type": "node",
-                            "data": {
-                                "name": "疯狂戴夫",
-                                "uin": "1298919732",
-                                "content": f"时间{time},{zombie}对{plants[tar]}造成{zb.damage}点伤害"
-                            }
-                        }
-                        )
+                        log.append(MessageSegment.text(f"时间{time},{zombie}对{plants[tar]}造成{zb.damage}点伤害"))
                         if lawn_plants[tar - 1].hp < 0:
-                            log.append({
-                                "type": "node",
-                                "data": {
-                                    "name": "疯狂戴夫",
-                                    "uin": "1298919732",
-                                    "content": f"时间{time},{plants[tar - 1]}被击败移出"
-                                }
-                            }
-                            )
+                            log.append(MessageSegment.text(f"时间{time},{plants[tar - 1]}被击败移出"))
                             lawn_plants.pop(tar - 1)
                             plant_pos.pop(tar - 1)
                             plants.pop(tar - 1)
@@ -962,26 +844,10 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                     if lawn_plants[tar].only_hurt_by_boss == 0 or (
                             lawn_plants[tar].only_hurt_by_boss == 1 and zb.is_gargantuar == 1):
                         lawn_plants[tar].hp -= zb.damage
-                        log.append({
-                            "type": "node",
-                            "data": {
-                                "name": "疯狂戴夫",
-                                "uin": "1298919732",
-                                "content": f"时间{time},{zombie}对{plants[tar]}造成{zb.damage if zb.jump == 0 else 0}点伤害。两次攻击间植物合计对{zombie}造成{all_damage}点伤害,{zombie}剩余血量为{zb.hp - all_damage}"
-                            }
-                        }
-                        )
+                        log.append(MessageSegment.text(f"时间{time},{zombie}对{plants[tar]}造成{zb.damage if zb.jump == 0 else 0}点伤害。两次攻击间植物合计对{zombie}造成{all_damage}点伤害,{zombie}剩余血量为{zb.hp - all_damage}"))
                         # 如果当前目标植物的耐久度小于0，则移除该植物
                         if lawn_plants[tar].hp < 0:
-                            log.append({
-                                "type": "node",
-                                "data": {
-                                    "name": "疯狂戴夫",
-                                    "uin": "1298919732",
-                                    "content": f"时间{time},{plants[tar]}被击败移出"
-                                }
-                            }
-                            )
+                            log.append(MessageSegment.text(f"时间{time},{plants[tar]}被击败移出"))
                             # 移除植物
                             lawn_plants.pop(tar)
                             plant_pos.pop(tar)
@@ -997,33 +863,12 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
         if iswin != 1 and zcnt == len(team):
             iswin = 2
         if iswin == 1:
-            log.append({"type": "node",
-                        "data": {"name": "疯狂戴夫",
-                                 "uin": "1298919732",
-                                 "content": [{"type": "text",
-                                              "data": {"text": f"入侵成功！成功吃掉对方脑子！"}},
-                                             {"type": "image",
-                                              "data": {"file": f"file:///{PVZ_ORI_PATH}/end.jpg"}}]}})
+            log.append(MessageSegment.text(f"入侵成功！成功吃掉对方脑子！")+MessageSegment.image(f"file:///{PVZ_ORI_PATH}/end.jpg"))
             break
         elif iswin == 2:
-            log.append({
-                "type": "node",
-                "data": {
-                    "name": "疯狂戴夫",
-                    "uin": "1298919732",
-                    "content": f"僵尸{zombie}被击败，所有僵尸均死去,入侵结束"
-                }
-            })
+            log.append(MessageSegment.text(f"僵尸{zombie}被击败，所有僵尸均死去,入侵结束"))
         elif iswin == 0:
-            log.append({
-                "type": "node",
-                "data": {
-                    "name": "疯狂戴夫",
-                    "uin": "1298919732",
-                    "content": f"{zombie}终于不堪重负被击败，但入侵仍在继续"
-                }
-            }
-            )
+            log.append(MessageSegment.text(f"{zombie}终于不堪重负被击败，但入侵仍在继续"))
 
     return log, iswin
 
@@ -1541,7 +1386,7 @@ async def _(event: GroupMessageEvent, state: T_State):
             else:
                 state["lawn"] = lawn
                 flag, users_2 = read_data(Path(bag_path))
-                if users_id in [x[0] for x in users_2]:
+                if user_id in [x[0] for x in users_2]:
                     state["bag_zombie"] = users_2[users_id.index(
                         str(event.user_id))][2].split(",")
                 else:
@@ -1591,14 +1436,14 @@ async def _(bot: Bot, event: GroupMessageEvent, state: T_State, team: str = ArgS
             # 一个一个入侵
             log, iswin = one_by_one(zombie_team, plants)
             # 发送数据
-            await bot.send_group_forward_msg(group_id=event.group_id, messages=log)
+            await send_forward_msg(bot, event, name="疯狂戴夫", uin=str(event.user_id), msgs=log)
 
 
 # 植物人机训练--------------------------------------------------------------------------
 
 
 @play_with_computer_zombie.handle()
-async def _(event: GroupMessageEvent, state: T_State, args: Message = CommandArg()):
+async def _(event: MessageEvent, state: T_State, args: Message = CommandArg()):
     flag, users = read_data(Path(lawn_path))
     users_id = [x[0] for x in users]
     user_id = str(event.user_id)
@@ -1616,7 +1461,7 @@ async def _(event: GroupMessageEvent, state: T_State, args: Message = CommandArg
 
 
 @play_with_computer_zombie.got("mode", prompt="请选择难度模式，可选择易，中，难，地狱四种不同级别的难度")
-async def _(bot: Bot, event: GroupMessageEvent, state: T_State, mode: str = ArgStr("mode")):
+async def _(bot: Bot, event: MessageEvent, state: T_State, mode: str = ArgStr("mode")):
     if mode in ["取消", "算了"]:
         await asyncio.sleep(1)
         await play_with_computer_zombie.finish("已取消操作...")
@@ -1631,7 +1476,7 @@ async def _(bot: Bot, event: GroupMessageEvent, state: T_State, mode: str = ArgS
             zombie_team = ["伽刚特尔", "橄榄球僵尸", "铁栅门僵尸", "小鬼僵尸"]
         lawn_plants = state["lawn"]
         log, iswin = one_by_one(zombie_team, lawn_plants)
-        await bot.send_group_forward_msg(group_id=event.group_id, messages=log)
+        await send_forward_msg(bot, event, name="疯狂戴夫", uin=str(event.user_id), msgs=log)
         await asyncio.sleep(0.5)
         if iswin == 1:
             await play_with_computer_zombie.finish(f"很遗憾您没能通过难度为{mode}的植物人机训练，此次出场的僵尸阵容为{','.join(zombie_team)}",
@@ -1648,7 +1493,7 @@ async def _(bot: Bot, event: GroupMessageEvent, state: T_State, mode: str = ArgS
 
 
 @play_with_computer_plant.handle()
-async def _(event: GroupMessageEvent, state: T_State, args: Message = CommandArg()):
+async def _(event: MessageEvent, state: T_State, args: Message = CommandArg()):
     flag, users = read_data(Path(bag_path))
     users_id = [x[0] for x in users]
     user_id = str(event.user_id)
@@ -1691,7 +1536,7 @@ async def _(state: T_State, mode: str = ArgStr("mode")):
 
 @play_with_computer_plant.got("team",
                               prompt="请选择您的僵尸入侵小队，僵尸之间用空格分割，最多选择三个僵尸，例如'普通僵尸 普通僵尸 普通僵尸'")
-async def _(bot: Bot, event: GroupMessageEvent, state: T_State, team: str = ArgStr("team")):
+async def _(bot: Bot, event: MessageEvent, state: T_State, team: str = ArgStr("team")):
     mode = state["mode"]
     if team in ["取消", "算了"]:
         await asyncio.sleep(1)
@@ -1722,7 +1567,7 @@ async def _(bot: Bot, event: GroupMessageEvent, state: T_State, team: str = ArgS
         # 一个一个入侵
         log, iswin = one_by_one(zombie_team, plants)
         # 发送数据
-        await bot.send_group_forward_msg(group_id=event.group_id, messages=log)
+        await send_forward_msg(bot, event, name="疯狂戴夫", uin=str(event.user_id), msgs=log)
         await asyncio.sleep(0.5)
         if iswin == 1:
             await play_with_computer_plant.finish(f"恭喜您成功通过难度为{mode}的僵尸人机训练",
@@ -1865,3 +1710,27 @@ def read_data(path: Path) -> (bool, list):
     except BaseException as e:
         print(e)
         return STATE_ERROR, []
+
+# 合并转发---------------------------------------------------------------------------------------
+
+
+async def send_forward_msg(
+    bot: Bot,
+    event: MessageEvent,
+    name: str,
+    uin: str,
+    msgs: List[Union[MessageSegment, Message]],
+) -> dict:
+    def to_json(msg: Union[MessageSegment, Message]):
+        return {"type": "node", "data": {"name": name, "uin": uin, "content": msg}}
+
+    messages = [to_json(msg) for msg in msgs]
+    print(messages)
+    if isinstance(event, GroupMessageEvent):
+        return await bot.call_api(
+            "send_group_forward_msg", group_id=event.group_id, messages=messages
+        )
+    else:
+        return await bot.call_api(
+            "send_private_forward_msg", user_id=event.user_id, messages=messages
+        )
